package pl.contium;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Element;

import org.hibernate.Session;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import java.util.HashSet;
import java.util.Set;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.junit.Assert.assertThat;


public class EhcacheHibernateTest {

	private InMemoryDb db;


	@Test
	public void shouldHaveOnlyMaxElementsInCacheAfterCreateAndDelete() throws Exception {

		// given
		Cache entityCache = CacheManager.getInstance().getCache(EntityImpl.REGION_NAME);
		long maxEntriesLocalHeap = 100;
		entityCache.getCacheConfiguration().setMaxEntriesLocalHeap(maxEntriesLocalHeap);

		// when
		createAndRemoveEntity(2000);
		createAndRemoveEntity(1);

		// then
		assertThat(entityCache.getCacheConfiguration().getMaxEntriesLocalHeap(), equalTo(maxEntriesLocalHeap));
		// BUG ?!
    for (Object o : entityCache.getKeys()) {
      final Element e = entityCache.get(o);
      System.out.println(entityCache.isPinned(o) + ": " + e);
    }
		assertThat(entityCache.getSize(), equalTo((int)maxEntriesLocalHeap));
	}

	private Set<Long> createAndRemoveEntity(int i) {

    Set<Long> ids = new HashSet<Long>(i, 1);

		Session session = db.createSession();

		for (int j = 0; j < i; j++) {
			EntityImpl entity = new EntityImpl();
			session.save(entity);
      ids.add(entity.getId());
			session.delete(entity);
		}

		session.flush();
//		session.clear();
		session.close();
    return ids;
	}
  
  private void loadAll(Set<Long> ids) {
    Session session = db.createSession();

    for (Long id : ids) {
      session.get(EntityImpl.class, id);
    }
    session.close();
  }

	@Before
	public void setUp() throws Exception {
		db = new InMemoryDb();
	}

	@After
	public void tearDown() {
		db.close();
	}
}
